#include <iostream>
#include <fstream>
#include <cstdlib>

#include <omp.h>
#include <TCanvas.h>
#include <TROOT.h>
#include <TApplication.h>

#include "Garfield/AvalancheMicroscopic.hh"
#include "Garfield/TrackHeed.hh"
#include "Garfield/ViewField.hh"
#include "Garfield/ViewCell.hh"
#include "Garfield/ComponentAnalyticField.hh"
#include "Garfield/MediumMagboltz.hh"
#include "Garfield/Sensor.hh"
#include "Garfield/ViewDrift.hh"
#include "Garfield/FundamentalConstants.hh"
#include "Garfield/DriftLineRKF.hh"
#include "Garfield/ViewSignal.hh"
#include "Garfield/Random.hh"

using namespace Garfield;

// Impulse response of the PASA.
double transfer(double t) {
  /* constexpr double risistance= 100 ;   //电阻设置
  constexpr double capacitance=  100;    //电容设置 */
  constexpr double tau=500;//ns
  constexpr double C = 200e-9;
  return (1/C)*exp(t/tau);
}
//原始电流信号打印到txt文件函数
void printsignaltotxt(Sensor& sensor,const std::string label,const int nTimebins,const double tstep,const double tmin,const double index){     
  char title[60];
  sprintf(title,"current_signal_%.2f.txt",index); 
  std::ofstream outfile;
  outfile.open(title,std::ios::out);
  outfile << "time(ns)     "<< "total current     "<< "electron current     " << "ion current\n";
  for(unsigned int i=0;i<nTimebins;++i){
    const double t=tstep/2+tmin+i*tstep;
    const double f=sensor.GetSignal(label,i);
    const double fe=sensor.GetElectronSignal(label,i);
    const double fion=sensor.GetIonSignal(label,i);
    outfile << t << "      "  << f << "      "  <<fe << "      "   << fion << "\n";
  }
}

//原始charge信号打印到txt文件函数
void printcharge(Sensor& sensor,const std::string label,const int nTimebins,const double tstep,const double tmin,const double index){     
  char title[60];
  sprintf(title, "charge_signal %.2f.txt",index); 
  std::ofstream outfile;
  outfile.open(title,std::ios::out);
  outfile << "time(ns)     "<< "total charge     "<< "electron charge   " << "ion charge\n";
  for(unsigned int i=0;i<nTimebins;++i){
    const double t=tstep/2+tmin+i*tstep;
    const double f=sensor.GetSignal(label,i);
    const double fe=sensor.GetElectronSignal(label,i);
    const double fion=sensor.GetIonSignal(label,i);
    outfile << t << "      "  << f << "      "  <<fe << "      "   << fion << "\n";
  }
}

//原始voltage信号打印到txt文件函数
void printvol(Sensor& sensor,const std::string label,const int nTimebins,const double tstep,const double tmin){     
  char title[60];
  sprintf(title, "voltage_signal.txt"); 
  std::ofstream outfile;
  outfile.open(title,std::ios::out);
  outfile << "time(ns)     "<< "total vol     "<< "electron vol   " << "ion vol\n";
  for(unsigned int i=0;i<nTimebins;++i){
    const double t=tstep/2+tmin+i*tstep;
    const double f=sensor.GetSignal(label,i);
    const double fe=sensor.GetElectronSignal(label,i);
    const double fion=sensor.GetIonSignal(label,i);
    outfile << t << "      "  << f << "      "  <<fe << "      "   << fion << "\n";
  }
}


int main(int argc, char * argv[]) {

  TApplication app("app", &argc, argv);
  
  // Periodicity (wire spacing) [cm].
  constexpr double period = 0.1;

  // Wire diameters [cm]直径
  // Gate wires.
  constexpr double dg = 0.008;//栅极丝直径cm

  // Voltage settings [V].
  // Gate wires.
  constexpr double vg = -642;//栅极电压
  // HV plane (drift field).
  constexpr double yHV = 0;
  constexpr double vHV = -5;
  // LV plane 8.5mm
  constexpr double yLV = 0.85;//阳极与阴极间距cm
  constexpr double vLV = -1000;
 
  // Setup the gas.
  MediumMagboltz gas;
  // Set the temperature [K] and pressure [Torr].
  gas.SetComposition("xe", 100.); //100%Xe
  gas.SetTemperature(293.15);
  gas.SetPressure(15*760.);
  gas.LoadGasFile("xe_15atm.gas");

  // Load the ion mobilities.
  const std::string path = std::getenv("GARFIELD_HOME");
  gas.LoadIonMobility(path + "/Data/IonMobility_Xe+_P12_Xe.txt");

  // Setup the electric field.
  ComponentAnalyticField cmp;
  cmp.SetMedium(&gas);
  cmp.SetPeriodicityX(4*period);

  // Add the gate wires.
  //constexpr double xg1 = 0.5 * period;
  //constexpr double xg2 = 0.75 * period;
  //栅极与阳极间距3mm
  constexpr double yg = 0.3;
  //cmp.AddWire(xg1, yg, dg, vg , "g+", 100., 50., 19.3, 1);
  //cmp.AddWire(xg2, yg, dg, vg , "g-", 100., 50., 19.3, 1);
 
  cmp.AddWire(-2.0 * period, yg, dg, vg, "g");
  cmp.AddWire(-1.5 * period, yg, dg, vg, "g");
  cmp.AddWire(-1.0 * period, yg, dg, vg, "g");
  cmp.AddWire(-0.5 * period, yg, dg, vg, "g");
  cmp.AddWire(0 * period,yg, dg, vg, "g");
  cmp.AddWire(+0.5 * period, yg, dg, vg, "g");
  cmp.AddWire(+1.0 * period, yg, dg, vg, "g");
  cmp.AddWire(+1.5 * period, yg, dg, vg, "g");
  cmp.AddWire(+2.0 * period, yg, dg, vg, "g"); 

  // cmp.AddWire(-3.5 * period, yg, dg, vg, "g4-");
  // cmp.AddWire(-2.5 * period, yg, dg, vg, "g3-");
  // cmp.AddWire(-1.5 * period, yg, dg, vg, "g2-");
  // cmp.AddWire(-0.5 * period, yg, dg, vg, "g1-");
  // cmp.AddWire(+0.5 * period, yg, dg, vg, "g1+");
  // cmp.AddWire(+1.5 * period, yg, dg, vg, "g2+");
  // cmp.AddWire(+2.5 * period, yg, dg, vg, "g3+");
  // cmp.AddWire(+3.5 * period, yg, dg, vg, "g4+"); 


  // Add the planes.
  cmp.AddPlaneY(yHV, vHV, "yang");
  cmp.AddPlaneY(yLV, vLV, "yin");

  // Request weighting-field calculation for the pad plane. 
  cmp.AddReadout("yang");
 
  // Make a sensor.
  Sensor sensor;
  sensor.AddComponent(&cmp);
  sensor.AddElectrode(&cmp, "yang");

  // Change the time window for less/better resolution in time (effect on convolution can be important).
  const double tstep = 10;      //信号采样步长，0.01微秒 （ns,s=1e9ns）
  const double tmin = 0;         //信号采样时间下限
  const double tmax=100000000;     //上限，根据结果调整  暂定1ms   
  const unsigned int nbins = (tmax-tmin)/tstep;   //信号采样时间间隔数 ,随机取样
  sensor.SetTimeWindow(tmin, tstep, nbins);
  constexpr double xmin = -2 * period;
  constexpr double xmax =  2 * period;
  sensor.SetArea(xmin, 0., -1., xmax, yLV, 1.);

  // Plot isopotential contours.
  ViewField fieldView;
  fieldView.SetSensor(&sensor);
  fieldView.SetArea(xmin, 0., xmax, yLV);
  //fieldView.SetVoltageRange(0., -1000.);
  fieldView.PlotContour();

  // //绘制电场
  // ViewField fieldView;
  // constexpr bool plotField = true;
  // if (plotField) {
  //   fieldView.SetComponent(&cmp);
  //   // Set the normal vector of the viewing plane (xy plane).
  //   fieldView.SetPlane(0, 0, 1, 0, 0, 0);
  //   fieldView.SetArea( xmin, 0., xmax, 0.2);
  //   fieldView.SetElectricFieldRange(0., 10000000.);
  //   TCanvas* cf = new TCanvas("Field", "", 600, 600);
  //   cf->SetLeftMargin(0.16);
  //   fieldView.SetCanvas(cf);
  //   fieldView.PlotContour("emag");
  // } 

  // Calculate ion drift lines using the RKF method.
  DriftLineRKF driftline;
  driftline.SetSensor(&sensor);

  // AvalancheMicroscopic aval;
  // aval.SetSensor(&sensor);
  
  // Plot the drift line.
  ViewDrift driftView;
  // Comment this out when calculating many drift lines.
  driftline.EnablePlotting(&driftView);
 
  //Set up Heed.设置入射粒子//考虑尝试光子
  TrackHeed track;
  track.SetParticle("muon");
  track.SetEnergy(170.e9);
  track.SetSensor(&sensor);

  // omp_set_num_threads(32);    //并行计算线程数
  // #pragma omp parallel for

  const int nIons = 1000;
  for (int i = 0; i < nIons; i++) {
    // Sample the starting point of the ion around the sense wire.
    const double x0 = 0.3 * RndmUniformPos()-0.15;
    const double y0 = 0.85 * RndmUniformPos();//注意修改
    driftline.DriftIon(x0,y0,0.,0.);
    //driftline.DriftElectron(x0,y0,0.,0.);
    // if(y0<0.7 || y0>0.08){
    //   driftline.DriftIon(x0, y0, 0, 0);
    //   double t0, e0, dx0, dy0, dz0;
    //   aval.DriftElectron(x0,y0,0.,0.,e0,dx0,dy0, dz0);
    //   std::cout << "lizi"<<" "<< i <<"is finished.\n";
    // } 
  }


  //光子入射
  // TrackHeed track;
  // track.SetSensor(&sensor);
  // track.EnableElectricField();
  // const double x0 = 0.6* RndmUniformPos()-0.2;
  // const double y0 = 0.2* RndmUniformPos();
  // const double egamma = 1.17e6;//1.17MeV
  // track.TransportPhoton(x0, y0, z0, t0, egamma, 0., 0., 1., ne);
  // for (int k = 0; k < ne; ++k) {
  //   double xe = 0., ye = 0., ze = 0., te = 0., ee = 0.;
  //   double dx = 0., dy = 0., dz = 0.;
  //   track.GetElectron(k, xe, ye, ze, te, ee, dx, dy, dz);
  //   driftline.DriftIon(xe, ye,0.,0.);
  //   driftline.DriftElectron(xe, ye,0.,0.);
  // }

  // constexpr unsigned int nEvents = 5;//入射粒子数
  // for (unsigned int i = 0; i < nEvents; ++i) { 
  //   const double y0 = 0;//0.04+0.16* RndmUniformPos();
  //   const double x0 = -0.1;//0.3* RndmUniformPos()-0.15;
  //   track.NewTrack(x0, y0, 0, 0, -0.87, 0.5, 0.);//(x0, y0, z0) at time t0 in direction (dx0, dy0, dz0).
  //   double xc = 0., yc = 0., zc = 0., tc = 0., ec = 0., extra = 0.;
  //   int ne = 0, ni = 0 ,np = 0;
  //   while(track.GetCluster(xc, yc, zc, tc, ne, ni , np, ec, extra) ){
  //     for (unsigned int k = 0; k < ne; ++k){
  //       double x1, y1, z1, t1, e1, dx1, dy1, dz1;
  //       double x0, y0, z0, t0, e0, dx0, dy0, dz0;
  //       track.GetIon(k, x1, y1, z1, t1);
  //       //track.GetElectron(k,x0,y0,z0,t0,e0,dx0,dy0, dz0);
  //       driftline.DriftIon(x1, y1,0.,0.);
  //       //driftline.DriftElectron(x0,y0,0.,0.);
  //       std::cout << "ion and elec"<<" "<< k <<" "<<"cal is finished.\n";  
  //     }
  //   }
  //   std::cout << "partice"<<" "<< i <<" "<<"Calculation is finished.\n";  
  // }

  // Plot the drift lines on top of the cell layout.
  ViewCell cellView;
  cellView.SetComponent(&cmp);
  cellView.SetArea(xmin, 0., xmax, yLV);
  cellView.Plot2d();
  driftView.SetArea(xmin, 0., xmax, yLV);
  driftView.SetColourTracks(kGreen + 3); 
  driftView.SetColourIons(kYellow);
  driftView.SetColourElectrons(kRed);
  driftView.SetCanvas(cellView.GetCanvas());
  driftView.Plot(true, false); 


  // //阴极信号
  // ViewSignal signalyin;
  // signalyin.SetSensor(&sensor);
  // TCanvas cy1("Current", "", 800, 600);
  // signalyin.SetCanvas(&cy1);
  // signalyin.SetLabelY("signal [μA]");
  // signalyin.PlotSignal("yin",true,true,true);
  // //  电流信号文件保存
  // printsignaltotxt(sensor,"yin",nbins,tstep,tmin); 


  // // 阳极上电荷计算
  // TCanvas cy2("Charge", "", 800, 600);
  // sensor.IntegrateSignal("yin");
  // signalyin.SetCanvas(&cy2);
  // signalyin.SetLabelY("Induced Charge [fC]");
  // signalyin.PlotSignal("yin",true,true,true);

  // // 电荷信号文件保存
  // printcharge(sensor,"yin",nbins,tstep,tmin);

  // Plot the induced current.
  ViewSignal signalView;

  signalView.SetSensor(&sensor);
  TCanvas c1("Current", "", 800, 600);
  signalView.SetCanvas(&c1);
  signalView.SetLabelY("signal [μA]");
  signalView.PlotSignal("yang");
  //  电流信号文件保存
  printsignaltotxt(sensor,"yang",nbins,tstep,tmin,0); 


  // 阳极上电荷计算
  TCanvas c2("Charge", "", 800, 600);
  sensor.IntegrateSignal("yang");
  signalView.SetCanvas(&c2);
  signalView.SetLabelY("Induced Charge [fC]");
  signalView.PlotSignal("yang");

  // 电荷信号文件保存
  printcharge(sensor,"yang",nbins,tstep,tmin,0);

  // // 电压信号
  // sensor.SetTransferFunction(transfer);
  // constexpr bool fft = true;
  // sensor.ConvoluteSignals(fft);
  // TCanvas c3("c2", "", 800, 600);
  // signalView.SetCanvas(&c3);
  // signalView.SetLabelY("signal [mV]");
  // signalView.PlotSignal("yang");
  // //导出电压信号
  // printvol(sensor,"yang",nbins,tstep,tmin);  

  std::cout << "Calculation is finished.\n";    //计算完成标志

  app.Run(true);

}

